// Generated by CoffeeScript 1.3.1
(function() {
  var AffiliationsSynchronization, ConfigSynchronization, ItemsSynchronization, NS, PaginatedSynchronization, RSM, SubscriptionsSynchronization, Synchronization, async, errors, logger, rsmWalk, syncQueue,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor; child.__super__ = parent.prototype; return child; };

  logger = require('./logger').makeLogger('sync');

  async = require('async');

  RSM = require('./xmpp/rsm');

  NS = require('./xmpp/ns');

  rsmWalk = require('./rsm_walk');

  errors = require('./errors');

  Synchronization = (function() {

    Synchronization.name = 'Synchronization';

    function Synchronization(router, node) {
      this.router = router;
      this.node = node;
      this.request = {
        operation: this.operation,
        node: this.node,
        writes: true
      };
    }

    Synchronization.prototype.run = function(t, cb) {
      var _this = this;
      return this.runRequest(function(err, results) {
        if (err) {
          return cb(err);
        }
        return _this.reset(t, function(err) {
          if (err) {
            return cb(err);
          }
          return _this.writeResults(t, results, cb);
        });
      });
    };

    Synchronization.prototype.runRequest = function(cb) {
      return this.router.runRemotely(this.request, cb);
    };

    return Synchronization;

  })();

  ConfigSynchronization = (function(_super) {

    __extends(ConfigSynchronization, _super);

    ConfigSynchronization.name = 'ConfigSynchronization';

    function ConfigSynchronization() {
      return ConfigSynchronization.__super__.constructor.apply(this, arguments);
    }

    ConfigSynchronization.prototype.operation = 'browse-node-info';

    ConfigSynchronization.prototype.reset = function(t, cb) {
      return t.resetConfig(this.node, cb);
    };

    ConfigSynchronization.prototype.writeResults = function(t, results, cb) {
      if (results.config) {
        return t.setConfig(this.node, results.config, cb);
      } else {
        return cb(new Error("No pubsub meta data form found"));
      }
    };

    return ConfigSynchronization;

  })(Synchronization);

  PaginatedSynchronization = (function(_super) {

    __extends(PaginatedSynchronization, _super);

    PaginatedSynchronization.name = 'PaginatedSynchronization';

    function PaginatedSynchronization() {
      PaginatedSynchronization.__super__.constructor.apply(this, arguments);
      this.request.rsm = new RSM.RSM();
    }

    PaginatedSynchronization.prototype.run = function(t, cb) {
      var _this = this;
      return rsmWalk(function(offset, cb2) {
        logger.debug("PaginatedSynchronization walk " + offset);
        _this.request.rsm.after = offset;
        return _this.runRequest(function(err, results) {
          var go;
          logger.debug("ranRequest " + (err || (results != null ? results.length : void 0)));
          if (err) {
            return cb2(err);
          }
          go = function(err) {
            if (err) {
              return cb2(err);
            }
            return _this.writeResults(t, results, function(err) {
              var nextOffset, _ref;
              if (err) {
                return cb2(err);
              }
              nextOffset = (_ref = results.rsm) != null ? _ref.last : void 0;
              return cb2(null, nextOffset);
            });
          };
          if (!_this.resetted) {
            _this.resetted = true;
            return _this.reset(t, go);
          } else {
            return go();
          }
        });
      }, cb);
    };

    return PaginatedSynchronization;

  })(Synchronization);

  ItemsSynchronization = (function(_super) {

    __extends(ItemsSynchronization, _super);

    ItemsSynchronization.name = 'ItemsSynchronization';

    function ItemsSynchronization() {
      return ItemsSynchronization.__super__.constructor.apply(this, arguments);
    }

    ItemsSynchronization.prototype.reset = function(t, cb) {
      return t.resetItems(this.node, cb);
    };

    ItemsSynchronization.prototype.operation = 'retrieve-node-items';

    ItemsSynchronization.prototype.writeResults = function(t, results, cb) {
      var _this = this;
      return async.forEach(results, function(item, cb2) {
        return t.writeItem(_this.node, item.id, item.el, cb2);
      }, cb);
    };

    return ItemsSynchronization;

  })(PaginatedSynchronization);

  SubscriptionsSynchronization = (function(_super) {

    __extends(SubscriptionsSynchronization, _super);

    SubscriptionsSynchronization.name = 'SubscriptionsSynchronization';

    function SubscriptionsSynchronization() {
      return SubscriptionsSynchronization.__super__.constructor.apply(this, arguments);
    }

    SubscriptionsSynchronization.prototype.reset = function(t, cb) {
      var _this = this;
      return t.resetSubscriptions(this.node, function(err, userListeners) {
        _this.userListeners = userListeners;
        return cb(err);
      });
    };

    SubscriptionsSynchronization.prototype.operation = 'retrieve-node-subscriptions';

    SubscriptionsSynchronization.prototype.run = function(t, cb) {
      var _this = this;
      return SubscriptionsSynchronization.__super__.run.call(this, t, function(err) {
        var cb2;
        if (err) {
          return cb(err);
        }
        cb2 = function() {
          return cb.apply(_this, arguments);
        };
        t.getNodeListeners(_this.node, function(err, listeners) {
          if (err) {
            logger.error("Cannot get node listeners: " + (err.stack || err));
          }
          if (!((listeners != null) && listeners.length > 0)) {
            return t.purgeNode(_this.node, cb2);
          }
        });
        return cb2();
      });
    };

    SubscriptionsSynchronization.prototype.writeResults = function(t, results, cb) {
      var _this = this;
      return async.forEach(results, function(item, cb2) {
        var listener;
        listener = _this.userListeners[item.user];
        return t.setSubscription(_this.node, item.user, listener, item.subscription, cb2);
      }, cb);
    };

    return SubscriptionsSynchronization;

  })(PaginatedSynchronization);

  AffiliationsSynchronization = (function(_super) {

    __extends(AffiliationsSynchronization, _super);

    AffiliationsSynchronization.name = 'AffiliationsSynchronization';

    function AffiliationsSynchronization() {
      return AffiliationsSynchronization.__super__.constructor.apply(this, arguments);
    }

    AffiliationsSynchronization.prototype.reset = function(t, cb) {
      var _this = this;
      return async.waterfall([
        t.validateNode(this.node), function(cb2) {
          return t.resetAffiliations(_this.node, cb2);
        }
      ], cb);
    };

    AffiliationsSynchronization.prototype.operation = 'retrieve-node-affiliations';

    AffiliationsSynchronization.prototype.writeResults = function(t, results, cb) {
      var _this = this;
      return async.forEach(results, function(item, cb2) {
        return t.setAffiliation(_this.node, item.user, item.affiliation, cb2);
      }, cb);
    };

    return AffiliationsSynchronization;

  })(PaginatedSynchronization);

  syncQueue = async.queue(function(task, cb) {
    var model, node, router, syncClass, synchronization;
    model = task.model, router = task.router, node = task.node, syncClass = task.syncClass;
    synchronization = new syncClass(router, node);
    return model.transaction(function(err, t) {
      if (err) {
        logger.error("sync transaction: " + (err.stack || err));
        return cb(err);
      }
      return synchronization.run(t, function(err) {
        if (err) {
          return t.rollback(function() {
            return cb(err);
          });
        } else {
          return t.commit(function(err) {
            return cb(err);
          });
        }
      });
    });
  }, 1);

  exports.syncNode = function(router, model, node, cb) {
    logger.debug("syncNode " + node);
    return async.forEachSeries([ConfigSynchronization, ItemsSynchronization, SubscriptionsSynchronization, AffiliationsSynchronization], function(syncClass, cb2) {
      return syncQueue.push({
        router: router,
        model: model,
        node: node,
        syncClass: syncClass
      }, cb2);
    }, function(err) {
      if (err && err.constructor === errors.SeeLocal) {
        logger.debug("Omitted syncing local node " + node);
        return typeof cb === "function" ? cb() : void 0;
      } else if (err) {
        logger.error("sync " + node + ": " + err);
        return typeof cb === "function" ? cb(err) : void 0;
      } else {
        logger.info("synced " + node);
        return typeof cb === "function" ? cb() : void 0;
      }
    });
  };

  exports.setup = function(jobs) {
    return syncQueue.concurrency = jobs;
  };

}).call(this);
